#include "I2C_Slave.h"
#include "PowerHz.h"
#include "Triac.h"

const uint8_t I2C_Addr = 0x10;  //  I2C.

enum
{
    CMD_SetPower = 1, //     (0-10000).
    CMD_GetPower,     //    .
    CMD_GetHz001      //      * 100.  50.2     5020.
};


/*
____________________________________________________________________________________________
|       |    (1 )  |                                              |
|__________________________________________________________________________________________|
| SetPower (w) |   (0 - 3) (w) |    (w) |    (w)  |
|------------------------------------------------------------------------------------------|
| GetPower (w) |   (0 - 3) (w) |    (r) |    (r)  |
|------------------------------------------------------------------------------------------|
| GetHz001 (w) |        ----          |    (r) |    (r)  |
--------------------------------------------------------------------------------------------

   .
 (w) -  . (r) -  .

*/

void I2C_Slave_Cnf(void)
{
    //   .
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    //   /.
    GPIO_InitTypeDef gpio;
    GPIO_StructInit(&gpio);
    gpio.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10; //   9  10.
    gpio.GPIO_Mode = GPIO_Mode_AF;   //  .
    gpio.GPIO_OType = GPIO_OType_OD; //    .
    //gpio.GPIO_PuPd = GPIO_PuPd_UP;   //     .
    GPIO_Init(GPIOA, &gpio);         //  GPIOA.

    //    9  10  GPIOA -   I2C.
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_4);  // SCL.
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_4); // SDA.


    //  I2C.
    RCC_I2CCLKConfig(RCC_I2C1CLK_SYSCLK); // I2C   .

    I2C_InitTypeDef i2c;
    I2C_StructInit(&i2c);
    i2c.I2C_OwnAddress1 = I2C_Addr; //  I2C.
    i2c.I2C_Ack = I2C_Ack_Enable;
    //i2c.I2C_Timing = 0x0090194B; // 400  clock, 100  , 100  .   48 .
    i2c.I2C_Timing = 0x10805D88; // 100  clock, 100  , 100  .   48 .
    i2c.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit; // 7-  .
    I2C_Init(I2C1, &i2c);

    I2C_StretchClockCmd(I2C1, ENABLE);

    //    I2C.
    I2C_ITConfig(I2C1, I2C_IT_ERRI | I2C_IT_STOPI | I2C_IT_ADDRI | I2C_IT_RXI  | I2C_IT_TXI, ENABLE);

    NVIC_InitTypeDef nvic;
    nvic.NVIC_IRQChannel = I2C1_IRQn;
    nvic.NVIC_IRQChannelPriority = 2;
    nvic.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&nvic);
}


void I2C1_IRQHandler(void) //   I2C.
{
    volatile static uint8_t InData[10], OutData[10];
    static uint8_t Cmd=0, Data=0, InBytes=10, OutBytes=0;
    static uint8_t InCount=0, OutCount=0;

    uint8_t ClearData=0;

    if (I2C_GetITStatus(I2C1, I2C_IT_RXNE)) //    .
    {
        uint8_t Byte = I2C_ReceiveData(I2C1);

        if (Cmd == 0) //  .
            Cmd = Byte;
        else if (InCount < InBytes)        //  .
        //else if (InCount<10)
            InData[InCount++] = Byte;

    }


    if (I2C_GetITStatus(I2C1, I2C_IT_TXIS)) //    .
    {
        I2C_ClearITPendingBit(I2C1, I2C_IT_TXIS);
        if (OutCount < OutBytes)
            I2C_SendData(I2C1, OutData[OutCount++]);
        else I2C_SendData(I2C1, 0);
    }


    if (I2C_GetITStatus(I2C1, I2C_IT_ADDR)) //    .
    {
        I2C_ClearITPendingBit(I2C1, I2C_IT_ADDR);

        switch (Cmd)
        {
        case CMD_SetPower: //     (0-10000).

            break;
        case CMD_GetPower: //    .
            {
                OutBytes = 2;
                (*(uint16_t*)&OutData[0]) = Triac_GetChannel(InData[0]);
            }
            break;
        case CMD_GetHz001: //   .
            OutBytes = 2;
            (*(uint16_t*)&OutData[0]) = PowerHz_GetHz001();
            break;
        default:
            ClearData=1; //  .
        }
    }

    if (I2C_GetITStatus(I2C1, I2C_IT_STOPF)) //  "".
    {
        I2C_ClearITPendingBit(I2C1, I2C_IT_STOPF);

        if (Cmd)
        {
            switch (Cmd)
            {
            case CMD_SetPower: //     (0-10000).
                if (InCount>=3)
                  Triac_SetChannel(InData[0], (InData[2]<<8)|InData[1]);
                break;
            }
        }

        ClearData=1; //        .
        I2C_SoftwareResetCmd(I2C1);
    }


    if (I2C_GetITStatus(I2C1, I2C_IT_BERR) || I2C_GetITStatus(I2C1, I2C_IT_ARLO)) // .
    {
        I2C_ClearITPendingBit(I2C1, I2C_IT_BERR);
        I2C_ClearITPendingBit(I2C1, I2C_IT_ARLO);
        ClearData=1;
    }


    if (I2C_GetITStatus(I2C1, I2C_IT_PECERR))
    {
        I2C_ClearITPendingBit(I2C1,I2C_IT_PECERR);
    }


    if (I2C_GetITStatus(I2C1, I2C_IT_TIMEOUT))
    {
        I2C_ClearITPendingBit(I2C1, I2C_IT_TIMEOUT);
        //ClearData=1;
    }


    if (ClearData)
    {
        Cmd = Data = InCount = OutBytes = OutCount = 0;
        InBytes = 10;
        for (uint8_t i=0; i<10; i++) InData[i] = OutData[i] = 0;
    }

}

