#include "PowerHz.h"
#include "Triac.h"

//   .

//#define TIMER_1
//#define TIMER_2
#define TIMER_3

#if defined(TIMER_1)

  #define TIMER TIM1
  #define TIMER_RCC RCC_APB2PeriphClockCmd(RCC_APB2ENR_TIM1EN, ENABLE)
  #define TIMER_IT TIM1_CC_IRQn
  #define TIMER_IT_Funct TIM1_CC_IRQHandler
  #define TIM_IT_Ch_Timeout TIM_IT_CC3
  #define GPIO_PIN GPIO_Pin_12
  #define GPIO_PORT GPIOA
  #define AF_PinSource GPIO_PinSource12
  #define AF_Num GPIO_AF_2

  #define DMA_Channel DMA1_Channel2
  #define DMA_IT_HT_FLAG DMA1_IT_HT2
  #define DMA_IT_TC_FLAG DMA1_IT_TC2
  #define DMA_IT_Funct DMA1_Channel2_3_IRQHandler
  #define DMA_Channel_IT DMA1_Channel2_3_IRQn
  #define DMA_Peripheral_DataSize DMA_PeripheralDataSize_HalfWord

#elif defined(TIMER_2)

  #define TIMER TIM2
  #define TIMER_RCC RCC_APB1PeriphClockCmd(RCC_APB1ENR_TIM2EN, ENABLE)
  #define TIMER_IT TIM2_IRQn
  #define TIMER_IT_Funct TIM2_IRQHandler
  #define TIM_IT_Ch_Timeout TIM_IT_CC3
  #define GPIO_PIN GPIO_Pin_0
  #define GPIO_PORT GPIOA
  #define AF_PinSource GPIO_PinSource0
  #define AF_Num GPIO_AF_2

  #define DMA_Channel DMA1_Channel5
  #define DMA_IT_HT_FLAG DMA1_IT_HT5
  #define DMA_IT_TC_FLAG DMA1_IT_TC5
  #define DMA_IT_Funct DMA1_Channel4_5_IRQHandler
  #define DMA_Channel_IT DMA1_Channel4_5_IRQn
  #define DMA_Peripheral_DataSize DMA_PeripheralDataSize_HalfWord

#elif defined(TIMER_3)

  #define TIMER TIM3
  #define TIMER_RCC RCC_APB1PeriphClockCmd(RCC_APB1ENR_TIM3EN, ENABLE)
  #define TIMER_IT TIM3_IRQn
  #define TIMER_IT_Funct TIM3_IRQHandler
  #define TIM_IT_Ch_Timeout TIM_IT_CC3
  #define GPIO_PIN GPIO_Pin_6
  #define GPIO_PORT GPIOA
  #define AF_PinSource GPIO_PinSource6
  #define AF_Num GPIO_AF_1

  #define DMA_Channel DMA1_Channel4
  #define DMA_IT_HT_FLAG DMA1_IT_HT4
  #define DMA_IT_TC_FLAG DMA1_IT_TC4
  #define DMA_IT_Funct DMA1_Channel4_5_IRQHandler
  #define DMA_Channel_IT DMA1_Channel4_5_IRQn
  #define DMA_Peripheral_DataSize DMA_PeripheralDataSize_HalfWord

#else
  #error "TIMER"
#endif

#define DMA_Buff_Size 100 //      .

extern uint32_t SystemCoreClock;

static volatile uint16_t DMA_CCR1_Buff[DMA_Buff_Size * 2]; //   DMA.
static volatile uint8_t Err_DMA_Transaction_flag = 0; //  0 -     .
static volatile uint8_t Err_DMA_Transaction_count = 0;
static volatile uint8_t OffChannels_Flag = 0;
static volatile uint16_t PowerTime_us = 10000; //    .


uint16_t PowerHz_GetTime_us(void) //     .
{
  return PowerTime_us;
}

uint16_t PowerHz_GetHz001(void) //      (0.01 ).
{
    return (uint16_t) (((uint32_t)100000000/2)/PowerTime_us);
}

//  .
void PowerHz_Cnf(void)
{
    //  .
    GPIO_InitTypeDef GPIO;
    TIM_TimeBaseInitTypeDef Tim;
    TIM_ICInitTypeDef TIM_IC;
    TIM_OCInitTypeDef TIM_OC;
    DMA_InitTypeDef Dma;
    NVIC_InitTypeDef nvic;

    //     .
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
    TIMER_RCC;

    //   /.
    GPIO_StructInit(&GPIO);
    GPIO.GPIO_Pin = GPIO_PIN;
    GPIO.GPIO_Mode = GPIO_Mode_AF;
    GPIO_Init(GPIO_PORT, &GPIO);

    //     /.
    GPIO_PinAFConfig(GPIO_PORT, AF_PinSource, AF_Num);

    //  .
    TIM_DeInit(TIMER);
    TIM_TimeBaseStructInit(&Tim);
    Tim.TIM_Prescaler = (SystemCoreClock / 1000000) - 1; //    1 .
    Tim.TIM_Period    = 65535;
    TIM_TimeBaseInit(TIMER, &Tim); //  .

    //   .
    TIM_ICStructInit(&TIM_IC);
    TIM_IC.TIM_Channel = TIM_Channel_1; //  
    TIM_IC.TIM_ICPolarity = TIM_ICPolarity_Rising; //   .
    TIM_IC.TIM_ICSelection = TIM_ICSelection_DirectTI; //   
    TIM_IC.TIM_ICPrescaler = TIM_ICPSC_DIV1; //  
    TIM_IC.TIM_ICFilter = 0; //  
    TIM_PWMIConfig(TIMER, &TIM_IC); //   .

    //   .
    TIM_SelectInputTrigger(TIMER, TIM_TS_TI1FP1);//    :  1.
    TIM_SelectSlaveMode(TIMER, TIM_SlaveMode_Reset); //       .
    TIM_SelectMasterSlaveMode(TIMER, TIM_MasterSlaveMode_Enable); //    .

    //  ,  .
    TIM_OCStructInit(&TIM_OC);
    TIM_OC.TIM_Pulse = 65530;
    TIM_OC3Init(TIMER, &TIM_OC);
     //       3 .
    TIM_ITConfig(TIMER, TIM_IT_CC3, ENABLE);
    TIM_ClearFlag(TIMER, TIM_FLAG_CC3);


    //  DMA.
    DMA_DeInit(DMA_Channel);
    DMA_StructInit(&Dma);

    Dma.DMA_PeripheralBaseAddr = (uint32_t) &(TIMER->CCR1); //  .
    Dma.DMA_MemoryBaseAddr = (uint32_t) DMA_CCR1_Buff;      //    ().
    Dma.DMA_DIR = DMA_DIR_PeripheralSRC;                    //  - ,   - .
    Dma.DMA_BufferSize = DMA_Buff_Size * 2;                 //    .
    Dma.DMA_PeripheralInc = DMA_PeripheralInc_Disable;      //    .
    Dma.DMA_MemoryInc = DMA_MemoryInc_Enable;               //   .
    Dma.DMA_PeripheralDataSize = DMA_Peripheral_DataSize;   //   .
    Dma.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;   //    (HalfWord - 2 ).
    Dma.DMA_Mode = DMA_Mode_Circular;                       //   ( ).
    Dma.DMA_Priority = DMA_Priority_High;                   //   DMA.
    Dma.DMA_M2M = DMA_M2M_Disable;                          //       .
    DMA_Init(DMA_Channel, &Dma);                            //   DMA.

    TIM_DMACmd(TIMER, TIM_DMA_CC1, ENABLE); //   DMA   1 .

     //       .
    DMA_ITConfig(DMA_Channel, DMA_IT_HT | DMA_IT_TC, ENABLE);

    DMA_Cmd(DMA_Channel, ENABLE); //   DMA.
    TIM_Cmd(TIMER, ENABLE);       //  .

    nvic.NVIC_IRQChannel = TIMER_IT;  //   .
    nvic.NVIC_IRQChannelCmd = ENABLE; //  .
    nvic.NVIC_IRQChannelPriority = 2; //   (0 - ; 3 - ).
    NVIC_Init(&nvic);

    nvic.NVIC_IRQChannel = DMA_Channel_IT; //   DMA.
    NVIC_Init(&nvic);

}

 //   Count   .
//__attribute__((optimize("-Ofast")))
static uint16_t PowerHz_Calc(volatile uint16_t *Buff, uint16_t Count)
{
    uint16_t i;
    uint32_t Res = 0;
    for (i=4; i<Count; i++) //           .
    {
        Res += Buff[i];
    }
    Res = Res / (Count - 4);
    return (uint16_t) Res;
}


void DMA_IT_Funct(void) //  DMA
{
    uint16_t Us=0; //    .

    if (DMA_GetITStatus(DMA_IT_HT_FLAG)) //   .
    {
        DMA_ClearITPendingBit(DMA_IT_HT_FLAG);

       if (Err_DMA_Transaction_flag == 0)
          Us = PowerHz_Calc(&DMA_CCR1_Buff[0], DMA_Buff_Size);
       else Err_DMA_Transaction_flag = 0;
    }
    else if (DMA_GetITStatus(DMA_IT_TC_FLAG)) //   .
    {
        DMA_ClearITPendingBit(DMA_IT_TC_FLAG);

       if (Err_DMA_Transaction_flag == 0)
          Us = PowerHz_Calc(&DMA_CCR1_Buff[DMA_Buff_Size], DMA_Buff_Size);
       else Err_DMA_Transaction_flag = 0;
    }

    //       83 - 125  ( 41 - 62 ).
    if ((Us>=8000) && (Us<=12000))
    {
        Err_DMA_Transaction_count=0;
        PowerTime_us = Us;
        Triac_Update(Us);
    }

    if (OffChannels_Flag) //    .
    {
        OffChannels_Flag = 0; //  .
        Triac_OnOff_Channel(ENABLE);
    }
}


void TIMER_IT_Funct(void) //  .
{
    if (TIM_GetITStatus(TIMER, TIM_IT_Ch_Timeout) != RESET)
    {
        TIM_ClearITPendingBit(TIMER, TIM_IT_Ch_Timeout);
        Err_DMA_Transaction_flag = 1; //    .

        if (Err_DMA_Transaction_count>=4) // -    .
        {
            if (OffChannels_Flag == 0)
            {
                OffChannels_Flag = 1; //   .
                Triac_OnOff_Channel(DISABLE);
            }

            PowerTime_us = 10000;
            Triac_Update(PowerTime_us);
        }
        else Err_DMA_Transaction_count++;
    }
}
