#include "Triac.h"
#include "PowerHz.h"


#define TIMER TIM2
#define TIMER_RCC RCC_APB1PeriphClockCmd(RCC_APB1ENR_TIM2EN, ENABLE)

#define GPIO_PINS (GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3)
#define GPIO_PORT GPIOA
#define AF_Pins GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_2);   /*   . */ \
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_2);  /*  2  -   . */ \
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_2); /*  3  -   . */ \
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_2);   /*  4  -   . */



extern uint32_t SystemCoreClock;
static volatile uint16_t ChannelsData[4]={0, 0, 0, 0};
static volatile uint16_t PowerTime_us = 10000; //    .

//        .
// Value -    0.01 %.
// Power_us -    .
static uint16_t Calc(int16_t Value, uint16_t Power_us)
{
    if ((Value>0) && (Power_us>0))
    {
        Value = (int16_t)((float)Value / (10000.0f / Power_us));

        if (Value>0)
        {
            uint16_t Arr = TIMER->ARR;
            if (Value > Arr) Value = Arr;
            else if (Value < 0) Value = 0;
        }
        else Value = 0;
    }

    return (uint16_t)Value;
}


//      .
void Triac_Update(uint16_t Power_us)
{
   //        0.1 ,   .
   if (((Power_us-20)>PowerTime_us) || ((Power_us+20)<PowerTime_us))
   {
       PowerTime_us = Power_us;
       TIMER->ARR = Power_us - 2;

       if (TIMER != TIM2)
         TIMER->CCR1 = Calc(ChannelsData[0], Power_us);

       TIMER->CCR2 = Calc(ChannelsData[1], Power_us);
       TIMER->CCR3 = Calc(ChannelsData[2], Power_us);
       TIMER->CCR4 = Calc(ChannelsData[3], Power_us);
   }

}



//      0.01%.
// Channel -   (0 - 3).
// Value -    (0 - 10000).
void Triac_SetChannel(uint8_t Channel, uint16_t Value)
{
    if (Channel>3) return;

    if (Value>10000) Value = 10000;
    Value = 10000 - Value;
    ChannelsData[Channel] = Value;

    Value = Calc(Value, PowerTime_us);

    switch (Channel)
    {
    case 0:
        if (TIMER != TIM2)
          TIMER->CCR1 = Value;
        break;
    case 1:
        TIMER->CCR2 = Value;
        break;
    case 2:
        TIMER->CCR3 = Value;
        break;
    case 3:
        TIMER->CCR4 = Value;
        break;
    }
}


uint16_t Triac_GetChannel(uint8_t Channel)
{
    uint16_t Value=0;

    if (Channel<4)
    {
        Value = 10000 - ChannelsData[Channel];
    }


    return Value;
}

//uint16_t Triac_GetChannel(uint8_t Channel)
//{
//    uint16_t Value=0;
//
//    switch (Channel)
//    {
//    case 0:
//        if (TIMER != TIM2)
//          Value = TIMER->CCR1;
//        break;
//    case 1:
//        Value = TIMER->CCR2;
//        break;
//    case 2:
//        Value = TIMER->CCR3;
//        break;
//    case 3:
//        Value = TIMER->CCR4;
//        break;
//    }
//
//    return Value;
//}

void Triac_OnOff_Channel(FunctionalState OnState)
{
    if (OnState == 0)
        GPIO_ResetBits(GPIO_PORT, GPIO_PINS);

    //   /.
    GPIO_InitTypeDef gpio;
    GPIO_StructInit(&gpio);
    gpio.GPIO_Pin = GPIO_PINS;
    gpio.GPIO_Mode = (OnState == ENABLE) ? GPIO_Mode_AF : GPIO_Mode_OUT;
    gpio.GPIO_Speed = GPIO_Speed_10MHz;
    GPIO_Init(GPIO_PORT, &gpio);    //  GPIO.
}

//       .
void Triac_Cnf(void)
{
    PowerTime_us = PowerHz_GetTime_us();

    TIM_TimeBaseInitTypeDef Tim;
    TIM_OCInitTypeDef PWM;

    //     .
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
    TIMER_RCC;

    AF_Pins; //     /.

    Triac_OnOff_Channel(ENABLE); //  .

    //  .
    TIM_DeInit(TIMER);
    TIM_TimeBaseStructInit(&Tim);
    Tim.TIM_Prescaler = (SystemCoreClock / 1000000) - 1; //    1 .
    Tim.TIM_Period    = PowerTime_us-2; //  100   .
    TIM_TimeBaseInit(TIMER, &Tim);         //  .

    //   .
    TIM_OCStructInit(&PWM);
    PWM.TIM_Pulse = Tim.TIM_Period + 2;
    PWM.TIM_OCMode = TIM_OCMode_PWM1;
    PWM.TIM_OutputState = TIM_OutputState_Enable;
    PWM.TIM_OCPolarity = TIM_OCPolarity_Low;

    if (TIMER != TIM2) TIM_OC1Init(TIMER, &PWM);
    TIM_OC2Init(TIMER, &PWM);
    TIM_OC3Init(TIMER, &PWM);
    TIM_OC4Init(TIMER, &PWM);

    //   .
    TIM_ETRConfig(TIMER, TIM_ExtTRGPSC_OFF, TIM_ExtTRGPolarity_NonInverted, 2); //     (   ).
    //TIM_ETRConfig(TIMER, TIM_ExtTRGPSC_OFF, TIM_ExtTRGPolarity_Inverted, 2); //     (   ).
    TIM_SelectInputTrigger(TIMER, TIM_TS_ETRF);      //   .
    TIM_SelectSlaveMode(TIMER, TIM_SlaveMode_Reset); //    .
    TIM_SelectMasterSlaveMode(TIMER, TIM_MasterSlaveMode_Enable); //    .

    TIM_Cmd(TIMER, ENABLE); //  .

}
